@@ -50,7 +50,7 @@ pub const OP_TYPE_ACCELERATION_STRUCTURE_KHR: OpCode = Op::TypeAccelerationStruc
 pub const TYPE_RANGE: RangeInclusive<OpCode> = OP_TYPE_VOID..=OP_TYPE_FUNCTION;
 
 pub const OP_CONSTANT_TRUE: OpCode = Op::ConstantTrue as u32;
-// pub const OP_CONSTANT_FALSE: OpCode = Op::ConstantFalse as u32;
+pub const OP_CONSTANT_FALSE: OpCode = Op::ConstantFalse as u32;
 pub const OP_CONSTANT: OpCode = Op::Constant as u32;
 // pub const OP_CONSTANT_COMPOSITE: OpCode = Op::ConstantComposite as u32;
 // pub const OP_CONSTANT_SAMPLER: OpCode = Op::ConstantSampler as u32;
@@ -74,6 +74,47 @@ pub const OP_LOAD: OpCode = Op::Load as u32;
 pub const OP_STORE: OpCode = Op::Store as u32;
 // pub const OP_IN_BOUNDS_ACCESS_CHAIN: OpCode = Op::InBoundsAccessChain as u32;
 
+// Integral ops used in specialization.
+pub const OP_SCONVERT: OpCode = Op::SConvert as u32;
+pub const OP_UCONVERT: OpCode = Op::UConvert as u32;
+pub const OP_FCONVERT: OpCode = Op::FConvert as u32;
+pub const OP_SNEGATE: OpCode = Op::SNegate as u32;
+pub const OP_NOT: OpCode = Op::Not as u32;
+pub const OP_IADD: OpCode = Op::IAdd as u32;
+pub const OP_ISUB: OpCode = Op::ISub as u32;
+pub const OP_IMUL: OpCode = Op::IMul as u32;
+pub const OP_UDIV: OpCode = Op::UDiv as u32;
+pub const OP_SDIV: OpCode = Op::SDiv as u32;
+pub const OP_UMOD: OpCode = Op::UMod as u32;
+pub const OP_SREM: OpCode = Op::SRem as u32;
+pub const OP_SMOD: OpCode = Op::SMod as u32;
+pub const OP_SHIFT_RIGHT_LOGICAL: OpCode = Op::ShiftRightLogical as u32;
+pub const OP_SHIFT_RIGHT_ARITHMETIC: OpCode = Op::ShiftRightArithmetic as u32;
+pub const OP_SHIFT_LEFT_LOGICAL: OpCode = Op::ShiftLeftLogical as u32;
+pub const OP_BITWISE_OR: OpCode = Op::BitwiseOr as u32;
+pub const OP_BITWISE_XOR: OpCode = Op::BitwiseXor as u32;
+pub const OP_BITWISE_AND: OpCode = Op::BitwiseAnd as u32;
+pub const OP_VECTOR_SHUFFLE: OpCode = Op::VectorShuffle as u32;
+pub const OP_COMPOSITE_EXTRACT: OpCode = Op::CompositeExtract as u32;
+pub const OP_COMPOSITE_INSERT: OpCode = Op::CompositeInsert as u32;
+pub const OP_LOGICAL_OR: OpCode = Op::LogicalOr as u32;
+pub const OP_LOGICAL_AND: OpCode = Op::LogicalAnd as u32;
+pub const OP_LOGICAL_NOT: OpCode = Op::LogicalNot as u32;
+pub const OP_LOGICAL_EQUAL: OpCode = Op::LogicalEqual as u32;
+pub const OP_LOGICAL_NOT_EQUAL: OpCode = Op::LogicalNotEqual as u32;
+pub const OP_SELECT: OpCode = Op::Select as u32;
+pub const OP_IEQUAL: OpCode = Op::IEqual as u32;
+pub const OP_INOT_EQUAL: OpCode = Op::INotEqual as u32;
+pub const OP_ULESS_THAN: OpCode = Op::ULessThan as u32;
+pub const OP_SLESS_THAN: OpCode = Op::SLessThan as u32;
+pub const OP_UGREATER_THAN: OpCode = Op::UGreaterThan as u32;
+pub const OP_SGREATER_THAN: OpCode = Op::SGreaterThan as u32;
+pub const OP_ULESS_THAN_EQUAL: OpCode = Op::ULessThanEqual as u32;
+pub const OP_SLESS_THAN_EQUAL: OpCode = Op::SLessThanEqual as u32;
+pub const OP_UGREATER_THAN_EQUAL: OpCode = Op::UGreaterThanEqual as u32;
+pub const OP_SGREATER_THAN_EQUAL: OpCode = Op::SGreaterThanEqual as u32;
+pub const OP_QUANTIZE_TO_F16: OpCode = Op::QuantizeToF16 as u32;
+
 pub const OP_ATOMIC_LOAD: OpCode = Op::AtomicLoad as u32;
 pub const OP_ATOMIC_STORE: OpCode = Op::AtomicStore as u32;
 pub const OP_ATOMIC_EXCHANGE: OpCode = Op::AtomicExchange as u32;
