@@ -58,6 +58,98 @@ impl fmt::Debug for InterfaceLocation {
 
 /// Specialization constant ID.
 pub type SpecId = u32;
+#[derive(Default, Clone, Copy, Debug)]
+pub struct ConstantValue {
+    buf: [u8; 8],
+}
+impl TryFrom<&[u32]> for ConstantValue {
+    type Error = Error;
+    fn try_from(x: &[u32]) -> Result<Self> {
+        let mut buf: [u8; 8] = [0; 8];
+        match x.len() {
+            1 => {
+                let bytes = u32::to_ne_bytes(x[0]);
+                (&mut buf[0..4]).copy_from_slice(&bytes);
+            },
+            2 => {
+                let lower_bytes = u32::to_ne_bytes(x[0]);
+                let upper_bytes = u32::to_ne_bytes(x[1]);
+                (&mut buf[0..4]).copy_from_slice(&lower_bytes);
+                (&mut buf[4..8]).copy_from_slice(&upper_bytes);
+            },
+            _ => return Err(Error::UNSUPPORTED_CONST_VALUE),
+        }
+        Ok(ConstantValue { buf })
+    }
+}
+impl TryFrom<&[u8]> for ConstantValue {
+    type Error = Error;
+    fn try_from(x: &[u8]) -> Result<Self> {
+        let mut buf: [u8; 8] = [0; 8];
+        match x.len() {
+            4 => (&mut buf[0..4]).copy_from_slice(&x),
+            8 => (&mut buf[0..8]).copy_from_slice(&x),
+            _ => return Err(Error::UNSUPPORTED_CONST_VALUE),
+        }
+        Ok(ConstantValue { buf })
+    }
+}
+impl From<[u8; 4]> for ConstantValue {
+    fn from(x: [u8; 4]) -> Self {
+        let mut out = ConstantValue::default();
+        out.buf[0..4].copy_from_slice(&x);
+        out
+    }
+}
+impl From<[u8; 8]> for ConstantValue {
+    fn from(x: [u8; 8]) -> Self {
+        let mut out = ConstantValue::default();
+        out.buf[0..8].copy_from_slice(&x);
+        out
+    }
+}
+impl From<bool> for ConstantValue {
+    fn from(x: bool) -> Self {
+        let mut out = ConstantValue::default();
+        out.buf[0] = if x { 1 } else { 0 };
+        out
+    }
+}
+impl From<u32> for ConstantValue {
+    fn from(x: u32) -> Self { Self::from(u32::to_ne_bytes(x)) }
+}
+impl From<i32> for ConstantValue {
+    fn from(x: i32) -> Self { Self::from(i32::to_ne_bytes(x)) }
+}
+impl From<f32> for ConstantValue {
+    fn from(x: f32) -> Self { Self::from(f32::to_ne_bytes(x)) }
+}
+impl From<u64> for ConstantValue {
+    fn from(x: u64) -> Self { Self::from(u64::to_ne_bytes(x)) }
+}
+impl From<i64> for ConstantValue {
+    fn from(x: i64) -> Self { Self::from(i64::to_ne_bytes(x)) }
+}
+impl From<f64> for ConstantValue {
+    fn from(x: f64) -> Self { Self::from(f64::to_ne_bytes(x)) }
+}
+impl ConstantValue {
+    fn to_bool(&self) -> bool {
+        self.to_u64() != 0
+    }
+    fn to_s32(&self) -> i32 {
+        self.to_s64() as i32
+    }
+    fn to_u32(&self) -> u32 {
+        self.to_u64() as u32
+    }
+    fn to_s64(&self) -> i64 {
+        i64::from_ne_bytes(self.buf)
+    }
+    fn to_u64(&self) -> u64 {
+        u64::from_ne_bytes(self.buf)
+    }
+}
 
 /// Variable locator.
 #[derive(PartialEq, Eq, Hash, Clone, Copy, Debug)]
@@ -72,24 +164,16 @@ pub enum Locator {
 
 // Intermediate types used in reflection.
 
-/// Reflection intermediate of specialization constants.
+/// Reflection intermediate of constants and specialization constant.
 #[derive(Debug, Clone)]
-pub struct SpecConstantIntermediate<'a> {
-    /// Type of specialization constant.
-    pub ty_id: TypeId,
-    /// Default value of specialization constant.
-    pub value: &'a [u32],
+pub struct ConstantIntermediate {
+    /// Type of constant.
+    pub ty: Type,
+    /// Defined value of constant, or default value of specialization constant.
+    pub value: ConstantValue,
     /// Specialization constant ID, notice that this is NOT an instruction ID.
     /// It is used to identify specialization constants for graphics libraries.
-    pub spec_id: SpecId,
-}
-/// Reflection intermediate of constants.
-#[derive(Debug, Clone)]
-pub struct ConstantIntermediate<'a> {
-    /// Type of constant.
-    pub ty_id: InstrId,
-    /// Defined value of constant.
-    pub value: &'a [u32],
+    pub spec_id: Option<SpecId>,
 }
 
 /// Descriptor type matching `VkDescriptorType`.
@@ -474,14 +558,13 @@ impl std::ops::BitAnd<AccessType> for AccessType {
 pub struct ReflectIntermediate<'a> {
     entry_point_declrs: Vec<EntryPointDeclartion<'a>>,
     execution_mode_declrs: Vec<ExecutionModeDeclaration>,
-    spec_consts: Vec<SpecConstantIntermediate<'a>>,
     vars: Vec<Variable>,
 
     name_map: HashMap<(InstrId, Option<u32>), &'a str>,
     deco_map: HashMap<(InstrId, Option<u32>, u32), &'a [u32]>,
     ty_map: HashMap<TypeId, Type>,
     var_map: HashMap<VariableId, usize>,
-    const_map: HashMap<ConstantId, ConstantIntermediate<'a>>,
+    const_map: HashMap<ConstantId, ConstantIntermediate>,
     ptr_map: HashMap<TypeId, TypeId>,
     func_map: HashMap<FunctionId, FunctionIntermediate>,
     declr_map: HashMap<Locator, InstrId>,
@@ -545,8 +628,19 @@ impl<'a> ReflectIntermediate<'a> {
             .unwrap_or(DescriptorBinding(0, 0))
     }
     /// Get the type identified by `ty_id`.
-    pub fn get_ty(&self, ty_id: TypeId) -> Option<&Type> {
+    pub fn get_ty(&self, ty_id: TypeId) -> Result<&Type> {
         self.ty_map.get(&ty_id)
+            .ok_or(Error::TY_NOT_FOUND)
+    }
+    fn put_ty(&mut self, ty_id: TypeId, ty: Type) -> Result<()> {
+        use std::collections::hash_map::Entry::Vacant;
+        match self.ty_map.entry(ty_id) {
+            Vacant(entry) => {
+                entry.insert(ty);
+                Ok(())
+            },
+            _ => Err(Error::ID_COLLISION),
+        }
     }
     /// Get the variable identified by `var_id`.
     pub fn get_var(&self, var_id: VariableId) -> Option<&Variable> {
@@ -557,8 +651,33 @@ impl<'a> ReflectIntermediate<'a> {
     /// Get the constant identified by `const_id`. Specialization constants are
     /// also stored as constants. Array extents specified by specialization
     /// constants are not statically known.
-    pub fn get_const(&self, const_id: ConstantId) -> Option<&ConstantIntermediate> {
+    pub fn get_const(&self, const_id: ConstantId) -> Result<&ConstantIntermediate> {
         self.const_map.get(&const_id)
+            .ok_or(Error::CONST_NOT_FOUND)
+    }
+    fn put_const(&mut self, const_id: ConstantId, constant: ConstantIntermediate) -> Result<()> {
+        use std::collections::hash_map::Entry::Vacant;
+        match self.const_map.entry(const_id) {
+            Vacant(entry) => {
+                entry.insert(constant);
+                Ok(())
+            },
+            _ => Err(Error::ID_COLLISION),
+        }
+    }
+    fn put_lit_const(
+        &mut self,
+        const_id: ConstantId,
+        ty_id: TypeId,
+        value: ConstantValue,
+        spec_id: Option<SpecId>,
+    ) -> Result<()> {
+        let constant = ConstantIntermediate {
+            ty: self.get_ty(ty_id)?.clone(),
+            value,
+            spec_id,
+        };
+        self.put_const(const_id, constant)
     }
     /// Get the human-friendly name of an instruction result.
     pub fn get_name(&self, id: InstrId) -> Option<&'a str> {
@@ -782,48 +901,53 @@ impl<'a> ReflectIntermediate<'a> {
         Ok(())
     }
     fn populate_one_ty(&mut self, instr: &Instr<'a>) -> Result<()> {
-        use std::collections::hash_map::Entry::Vacant;
-        let (key, value) = match instr.opcode() {
-            OP_TYPE_FUNCTION => { return Ok(()) },
+        match instr.opcode() {
+            OP_TYPE_FUNCTION => {
+                Ok(())
+            },
             OP_TYPE_VOID => {
                 let op = OpTypeVoid::try_from(instr)?;
-                (op.ty_id, Type::Void())
+                self.put_ty(op.ty_id, Type::Void())
             },
             OP_TYPE_BOOL => {
                 let op = OpTypeBool::try_from(instr)?;
                 let scalar_ty = ScalarType::boolean();
-                (op.ty_id, Type::Scalar(scalar_ty))
+                self.put_ty(op.ty_id, Type::Scalar(scalar_ty))
             },
             OP_TYPE_INT => {
                 let op = OpTypeInt::try_from(instr)?;
                 let scalar_ty = ScalarType::int(op.nbyte >> 3, op.is_signed);
-                (op.ty_id, Type::Scalar(scalar_ty))
+                self.put_ty(op.ty_id, Type::Scalar(scalar_ty))
             },
             OP_TYPE_FLOAT => {
                 let op = OpTypeFloat::try_from(instr)?;
                 let scalar_ty = ScalarType::float(op.nbyte >> 3);
-                (op.ty_id, Type::Scalar(scalar_ty))
+                self.put_ty(op.ty_id, Type::Scalar(scalar_ty))
             },
             OP_TYPE_VECTOR => {
                 let op = OpTypeVector::try_from(instr)?;
-                if let Some(Type::Scalar(scalar_ty)) = self.get_ty(op.scalar_ty_id) {
+                if let Type::Scalar(scalar_ty) = self.get_ty(op.scalar_ty_id)? {
                     let vec_ty = VectorType::new(scalar_ty.clone(), op.nscalar);
-                    (op.ty_id, Type::Vector(vec_ty))
-                } else { return Err(Error::TY_NOT_FOUND); }
+                    self.put_ty(op.ty_id, Type::Vector(vec_ty))
+                } else {
+                    Err(Error::BROKEN_NESTED_TY)
+                }
             },
             OP_TYPE_MATRIX => {
                 let op = OpTypeMatrix::try_from(instr)?;
-                if let Some(Type::Vector(vec_ty)) = self.get_ty(op.vec_ty_id) {
+                if let Type::Vector(vec_ty) = self.get_ty(op.vec_ty_id)? {
                     let mat_ty = MatrixType::new(vec_ty.clone(), op.nvec);
-                    (op.ty_id, Type::Matrix(mat_ty))
-                } else { return Err(Error::TY_NOT_FOUND); }
+                    self.put_ty(op.ty_id, Type::Matrix(mat_ty))
+                } else {
+                    Err(Error::BROKEN_NESTED_TY)
+                }
             },
             OP_TYPE_IMAGE => {
                 let op = OpTypeImage::try_from(instr)?;
-                let scalar_ty = match self.get_ty(op.scalar_ty_id) {
-                    Some(Type::Scalar(scalar_ty)) => Some(scalar_ty.clone()),
-                    Some(Type::Void()) => None,
-                    _ => return Err(Error::TY_NOT_FOUND),
+                let scalar_ty = match self.get_ty(op.scalar_ty_id)? {
+                    Type::Scalar(scalar_ty) => Some(scalar_ty.clone()),
+                    Type::Void() => None,
+                    _ => return Err(Error::BROKEN_NESTED_TY),
                 };
                 let img_ty = if op.dim == Dim::DimSubpassData {
                     let arng = SubpassDataArrangement::from_spv_def(op.is_multisampled)?;
@@ -839,30 +963,28 @@ impl<'a> ReflectIntermediate<'a> {
                     let img_ty = ImageType::new(scalar_ty, unit_fmt, arng);
                     Type::Image(img_ty)
                 };
-                (op.ty_id, img_ty)
+                self.put_ty(op.ty_id, img_ty)
             },
             OP_TYPE_SAMPLER => {
                 let op = OpTypeSampler::try_from(instr)?;
                 // Note that SPIR-V doesn't discriminate color and depth/stencil
                 // samplers. `sampler` and `samplerShadow` means the same thing.
-                (op.ty_id, Type::Sampler())
+                self.put_ty(op.ty_id, Type::Sampler())
             },
             OP_TYPE_SAMPLED_IMAGE => {
                 let op = OpTypeSampledImage::try_from(instr)?;
-                if let Some(Type::Image(img_ty)) = self.get_ty(op.img_ty_id) {
+                if let Type::Image(img_ty) = self.get_ty(op.img_ty_id)? {
                     let sampled_img_ty = SampledImageType::new(img_ty.clone());
-                    (op.ty_id, Type::SampledImage(sampled_img_ty))
-                } else { return Err(Error::TY_NOT_FOUND); }
+                    self.put_ty(op.ty_id, Type::SampledImage(sampled_img_ty))
+                } else {
+                    Err(Error::BROKEN_NESTED_TY)
+                }
             },
             OP_TYPE_ARRAY => {
                 let op = OpTypeArray::try_from(instr)?;
-                let proto_ty = if let Some(proto_ty) = self.get_ty(op.proto_ty_id) {
-                    proto_ty
-                } else {
-                    return Ok(());
-                };
+                let proto_ty = if let Ok(x) = self.get_ty(op.proto_ty_id) { x } else { return Ok(()); };
 
-                let nrepeat = self.const_map.get(&op.nrepeat_const_id)
+                let nrepeat = self.get_const(op.nrepeat_const_id)?
                     // Some notes about specialization constants.
                     //
                     // Using specialization constants for array sizes might lead
@@ -872,55 +994,21 @@ impl<'a> ReflectIntermediate<'a> {
                     // behavior of `glslang` is to treat the specialization
                     // constants as normal constants, then I would say...
                     // probably it's fine to size array with them?
-                    .and_then(|constant| {
-                        if let Some(Type::Scalar(scalar_ty)) = self.get_ty(constant.ty_id) {
-                            if scalar_ty.nbyte() == 4 && scalar_ty.is_uint() {
-                                return constant.value.iter().next().cloned();
-                            }
-                        }
-                        None
-                    });
+                    .value
+                    .to_u32();
                 let stride = self.get_deco_u32(op.ty_id, Decoration::ArrayStride)
                     .map(|x| x as usize);
 
-                let arr_ty = if let Some(nrepeat) = nrepeat {
-                    if let Some(stride) = stride {
-                        ArrayType::new(&proto_ty, nrepeat, stride)
-                    } else {
-                        ArrayType::new_multibind(&proto_ty, nrepeat)
-                    }
+                let arr_ty = if let Some(stride) = stride {
+                    ArrayType::new(&proto_ty, nrepeat, stride)
                 } else {
-                    // We expect the constant is registered but we failed to
-                    // find it. It's possible that the SPIR-V generated a
-                    // forward reference to a result of `OpCompositeExtract` to
-                    // get a component of a specialized composite. One example
-                    // is to use workgroup size to declare arrays.
-                    //
-                    // Such behavior is observed in `glslangValidator`, built
-                    // with glslang 11.1; but AFAIK no longer with glslang 11.5
-                    // in which `OpSpecConstantOp` with `OpCompositeExtract`
-                    // is generated instead.
-                    //
-                    // Either way, if array size is a specialized value, we have
-                    // no idea about the actual size of the array through static
-                    // analysis. To the most possible extent, we assume the
-                    // SPIR-V input is valid so we also assume any missing
-                    // reference points to a forward-referenced instruction.
-                    if let Some(stride) = stride {
-                        ArrayType::new_unsized(&proto_ty, stride)
-                    } else {
-                        ArrayType::new_unsized_multibind(&proto_ty)
-                    }
+                    ArrayType::new_multibind(&proto_ty, nrepeat)
                 };
-                (op.ty_id, Type::Array(arr_ty))
+                self.put_ty(op.ty_id, Type::Array(arr_ty))
             },
             OP_TYPE_RUNTIME_ARRAY => {
                 let op = OpTypeRuntimeArray::try_from(instr)?;
-                let proto_ty = if let Some(proto_ty) = self.get_ty(op.proto_ty_id) {
-                    proto_ty
-                } else {
-                    return Ok(());
-                };
+                let proto_ty = if let Ok(x) = self.get_ty(op.proto_ty_id) { x } else { return Ok(()); };
                 let stride = self.get_deco_u32(op.ty_id, Decoration::ArrayStride)
                     .map(|x| x as usize);
                 let arr_ty = if let Some(stride) = stride {
@@ -928,7 +1016,7 @@ impl<'a> ReflectIntermediate<'a> {
                 } else {
                     ArrayType::new_unsized_multibind(&proto_ty)
                 };
-                (op.ty_id, Type::Array(arr_ty))
+                self.put_ty(op.ty_id, Type::Array(arr_ty))
             },
             OP_TYPE_STRUCT => {
                 let op = OpTypeStruct::try_from(instr)?;
@@ -936,7 +1024,7 @@ impl<'a> ReflectIntermediate<'a> {
                 let mut struct_ty = StructType::new(struct_name);
                 for (i, &member_ty_id) in op.member_ty_ids.iter().enumerate() {
                     let i = i as u32;
-                    let mut member_ty = if let Some(member_ty) = self.get_ty(member_ty_id) {
+                    let mut member_ty = if let Ok(member_ty) = self.get_ty(member_ty_id) {
                         member_ty.clone()
                     } else {
                         return Ok(());
@@ -979,7 +1067,7 @@ impl<'a> ReflectIntermediate<'a> {
                 }
                 // Don't have to shrink-to-fit because the types in `ty_map`
                 // won't be used directly and will be cloned later.
-                (op.ty_id, Type::Struct(struct_ty))
+                self.put_ty(op.ty_id, Type::Struct(struct_ty))
             },
             OP_TYPE_POINTER => {
                 let op = OpTypePointer::try_from(instr)?;
@@ -989,127 +1077,223 @@ impl<'a> ReflectIntermediate<'a> {
             },
             OP_TYPE_ACCELERATION_STRUCTURE_KHR => {
                 let op = OpTypeAccelerationStructureKHR::try_from(instr)?;
-                (op.ty_id, Type::AccelStruct())
+                self.put_ty(op.ty_id, Type::AccelStruct())
             },
             _ => return Err(Error::UNSUPPORTED_TY),
-        };
-        if let Vacant(entry) = self.ty_map.entry(key) {
-            entry.insert(value); Ok(())
-        } else { Err(Error::ID_COLLISION) }
-    }
-    fn populate_one_const(&mut self, instr: &Instr<'a>) -> Result<()> {
-        use std::collections::hash_map::Entry::Vacant;
-        if instr.opcode() == OP_CONSTANT {
-            let op = OpConstant::try_from(instr)?;
-            if let Vacant(entry) = self.const_map.entry(op.const_id) {
-                let constant = ConstantIntermediate {
-                    ty_id: op.ty_id,
-                    value: op.value,
-                };
-                entry.insert(constant);
-                Ok(())
-            } else { Err(Error::ID_COLLISION) }
-        } else {
-            Ok(())
         }
     }
-    fn populate_one_spec_const(&mut self, instr: &Instr<'a>) -> Result<()> {
-        use std::collections::hash_map::Entry::Vacant;
-        let (spec_const_id, constant, spec_const) = match instr.opcode() {
-            OP_SPEC_CONSTANT_TRUE => {
-                let op = OpSpecConstantTrue::try_from(instr)?;
-                let constant = ConstantIntermediate {
-                    ty_id: op.ty_id,
-                    value: &[1],
-                };
-                let spec_id = self.get_deco_u32(op.spec_const_id, Decoration::SpecId)
-                    .ok_or(Error::MISSING_DECO)?;
-                let spec_const = SpecConstantIntermediate {
-                    ty_id: constant.ty_id,
-                    value: &[1],
-                    spec_id,
-                };
-                (op.spec_const_id, constant, Some(spec_const))
+    fn populate_one_const(&mut self, instr: &Instr<'a>) -> Result<()> {
+        let op = OpConstantScalarCommonSPQ::try_from(instr)?;
+        match instr.opcode() {
+            OP_CONSTANT_TRUE => {
+                self.put_lit_const(op.const_id, op.ty_id, ConstantValue::from(true), None)
             },
-            OP_SPEC_CONSTANT_FALSE => {
-                let op = OpSpecConstantFalse::try_from(instr)?;
-                let constant = ConstantIntermediate {
-                    ty_id: op.ty_id,
-                    value: &[0],
-                };
-                let spec_id = self.get_deco_u32(op.spec_const_id, Decoration::SpecId)
-                    .ok_or(Error::MISSING_DECO)?;
-                let spec_const = SpecConstantIntermediate {
-                    ty_id: constant.ty_id,
-                    value: &[0],
-                    spec_id,
-                };
-                (op.spec_const_id, constant, Some(spec_const))
+            OP_CONSTANT_FALSE => {
+                self.put_lit_const(op.const_id, op.ty_id, ConstantValue::from(false), None)
             },
-            OP_SPEC_CONSTANT => {
-                let op = OpSpecConstant::try_from(instr)?;
-                let constant = ConstantIntermediate {
-                    ty_id: op.ty_id,
-                    value: op.value,
-                };
-                let spec_id = self.get_deco_u32(op.spec_const_id, Decoration::SpecId)
+            OP_CONSTANT => {
+                self.put_lit_const(op.const_id, op.ty_id, ConstantValue::try_from(op.value)?, None)
+            },
+            _ => Ok(()),
+        }
+    }
+    fn populate_one_spec_const_op(&mut self, instr: &Instr<'a>) -> Result<()> {
+        let op = OpSpecConstantHeadSPQ::try_from(instr)?;
+        match op.opcode {
+            OP_SCONVERT => {
+                let op = OpSpecConstantUnaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(a), None)
+            },
+            OP_UCONVERT => {
+                let op = OpSpecConstantUnaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(a), None)
+            },
+            OP_SNEGATE => {
+                let op = OpSpecConstantUnaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_s32();
+                let value = a.overflowing_neg().0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_NOT => {
+                let op = OpSpecConstantUnaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let value = a.overflowing_neg().0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_IADD => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a.overflowing_add(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_ISUB => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a.overflowing_sub(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_IMUL => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a.overflowing_mul(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_UDIV => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a.overflowing_div(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_SDIV => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_s32();
+                let b = self.get_const(op.b_id)?.value.to_s32();
+                let value = a.overflowing_div(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_UMOD => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a.overflowing_rem_euclid(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_SREM => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_s32();
+                let b = self.get_const(op.b_id)?.value.to_s32();
+                let value = a.overflowing_rem(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_SMOD => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_s32();
+                let b = self.get_const(op.b_id)?.value.to_s32();
+                let value = a.overflowing_rem_euclid(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_SHIFT_RIGHT_LOGICAL => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a.overflowing_shr(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            // Rust don't have a arithmetic shift.
+            //OP_SHIFT_RIGHT_ARITHMETIC => {}
+            OP_SHIFT_LEFT_LOGICAL => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a.overflowing_shl(b).0;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_BITWISE_OR => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a | b;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_BITWISE_XOR => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a ^ b;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_BITWISE_AND => {
+                let op = OpSpecConstantBinaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_u32();
+                let b = self.get_const(op.b_id)?.value.to_u32();
+                let value = a & b;
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            },
+            OP_SELECT => {
+                let op = OpSpecConstantTertiaryOpCommonSPQ::try_from(instr)?;
+                let a = self.get_const(op.a_id)?.value.to_bool();
+                let b = self.get_const(op.b_id)?.value;
+                let c = self.get_const(op.c_id)?.value;
+                let value = if a { b } else { c };
+                self.put_lit_const(op.spec_const_id, op.ty_id, ConstantValue::from(value), None)
+            }
+            OP_FCONVERT |
+            OP_VECTOR_SHUFFLE | OP_COMPOSITE_EXTRACT | OP_COMPOSITE_INSERT |
+            OP_LOGICAL_OR | OP_LOGICAL_AND | OP_LOGICAL_NOT |
+            OP_LOGICAL_EQUAL | OP_LOGICAL_NOT_EQUAL |
+            OP_IEQUAL | OP_INOT_EQUAL |
+            OP_ULESS_THAN | OP_SLESS_THAN |
+            OP_UGREATER_THAN | OP_SGREATER_THAN |
+            OP_ULESS_THAN_EQUAL | OP_SLESS_THAN_EQUAL |
+            OP_UGREATER_THAN_EQUAL | OP_SGREATER_THAN_EQUAL |
+            OP_QUANTIZE_TO_F16 => {
+                // Maybe these ops can be supported in the future, but seems
+                // unnecessary ATM.
+                Ok(())
+            },
+            _ => Err(Error::UNSUPPORTED_SPEC),
+        }
+    }
+    fn populate_one_spec_const(&mut self, instr: &Instr<'a>, cfg: &ReflectConfig) -> Result<()> {
+        match instr.opcode() {
+            OP_SPEC_CONSTANT_TRUE | OP_SPEC_CONSTANT_FALSE | OP_SPEC_CONSTANT => {
+                let op = OpConstantScalarCommonSPQ::try_from(instr)?;
+                let spec_id = self.get_deco_u32(op.const_id, Decoration::SpecId)
                     .ok_or(Error::MISSING_DECO)?;
-                let spec_const = SpecConstantIntermediate {
-                    ty_id: constant.ty_id,
-                    value: op.value,
-                    spec_id,
-                };
-                (op.spec_const_id, constant, Some(spec_const))
+
+                if let Some(x) = cfg.spec_values.get(&spec_id) {
+                    self.put_lit_const(op.const_id, op.ty_id, x.clone(), None)
+                } else {
+                    match instr.opcode() {
+                        OP_SPEC_CONSTANT_TRUE => {
+                            self.put_lit_const(op.const_id, op.ty_id, ConstantValue::from(true), Some(spec_id))
+                        },
+                        OP_SPEC_CONSTANT_FALSE => {
+                            self.put_lit_const(op.const_id, op.ty_id, ConstantValue::from(false), Some(spec_id))
+                        },
+                        OP_SPEC_CONSTANT => {
+                            self.put_lit_const(op.const_id, op.ty_id, ConstantValue::try_from(op.value)?, Some(spec_id))
+                        },
+                        _ => unreachable!(),
+                    }
+                }
             },
             // `SpecId` decorations will be specified to each of the
             // constituents so we don't have to register a
-            // `SpecConstantIntermediate` for the composite of them.
-            // `SpecConstantIntermediate` is registered only for those will be
+            // `ConstantIntermediate` for the composite of them.
+            // `ConstantIntermediate` is registered only for those will be
             // interacting with Vulkan.
             OP_SPEC_CONSTANT_COMPOSITE => {
-                let op = OpSpecConstantComposite::try_from(instr)?;
-                let constant = ConstantIntermediate {
-                    ty_id: op.ty_id,
-                    // Empty value to annotate a specialization constant. We
-                    // have nothing like a `SpecId` to access such
-                    // specialization constant so it's unnecesary to resolve
-                    // it's default value. Same applies to `OpSpecConstantOp`.
-                    value: &[] as &'static [u32],
-                };
-                (op.spec_const_id, constant, None)
+                //let op = OpSpecConstantComposite::try_from(instr)?;
+                //let constant = ConstantIntermediate {
+                //    // Empty value to annotate a specialization constant. We
+                //    // have nothing like a `SpecId` to access such
+                //    // specialization constant so it's unnecesary to resolve
+                //    // it's default value. Same applies to `OpSpecConstantOp`.
+                //    value: &[] as &'static [u32],
+                //    spec_id: None,
+                //};
+                //(op.spec_const_id, constant)
+                Ok(())
             },
-            // Similar to `OpConstantComposite`, we don't register
-            // specialization constants for `OpSpecConstantOp` results, neither
-            // the validity of the operations because they are out of SPIR-Q's
-            // duty.
-            //
-            // NOTE: In some cases you might want to use specialized workgroup
-            // size to allocate shared memory or other on-chip memory with this,
-            // that's possible, but still be aware that specialization constants
-            // CANNOT be used to specify any STRUCTURED memory objects like UBO
-            // and SSBO, because the stride and offset decorations are
-            // precompiled as a part of the SPIR-V binary meta.
             OP_SPEC_CONSTANT_OP => {
-                let op = OpSpecConstantOp::try_from(instr)?;
-                let constant = ConstantIntermediate {
-                    ty_id: op.ty_id,
-                    value: &[] as &'static [u32],
-                };
-                (op.spec_const_id, constant, None)
+                let res = self.populate_one_spec_const_op(instr);
+                // Tolerate unfound constants during constant capture. The
+                // absence of constant is only critical when it's used as array
+                // size.
+                if res == Err(Error::CONST_NOT_FOUND) {
+                    return Ok(())
+                }
+                res
             },
-            _ => return Err(Error::UNSUPPORTED_SPEC),
-        };
-
-        if let Vacant(entry) = self.const_map.entry(spec_const_id) {
-            entry.insert(constant);
-        } else { return Err(Error::ID_COLLISION) }
-        if let Some(spec_const) = spec_const {
-            let locator = Locator::SpecConstant(spec_const.spec_id);
-            self.declr_map.insert(locator, spec_const_id);
-            self.spec_consts.push(spec_const);
+            _ => Err(Error::UNSUPPORTED_SPEC),
         }
-
-        Ok(())
     }
     fn populate_one_var(&mut self, instr: &Instr<'a>) -> Result<()> {
         fn extract_proto_ty<'a>(ty: &Type) -> Result<(u32, Type)> {
@@ -1130,7 +1314,7 @@ impl<'a> ReflectIntermediate<'a> {
         let op = OpVariable::try_from(instr)?;
         let ty_id = self.access_chain(op.ty_id)
             .ok_or(Error::BROKEN_ACCESS_CHAIN)?;
-        let ty = if let Some(ty) = self.get_ty(ty_id) {
+        let ty = if let Ok(ty) = self.get_ty(ty_id) {
             ty
         } else {
             // If a variable is declared based on a unregistered type, very
@@ -1265,7 +1449,7 @@ impl<'a> ReflectIntermediate<'a> {
 
         Ok(())
     }
-    fn populate_defs(&mut self, instrs: &'_ mut Peekable<Instrs<'a>>) -> Result<()> {
+    fn populate_defs(&mut self, instrs: &'_ mut Peekable<Instrs<'a>>, cfg: &ReflectConfig) -> Result<()> {
         // type definitions always follow decorations, so we don't skip
         // instructions here.
         while let Some(instr) = instrs.peek() {
@@ -1277,7 +1461,7 @@ impl<'a> ReflectIntermediate<'a> {
             } else if CONST_RANGE.contains(&opcode) {
                 self.populate_one_const(instr)?;
             } else if SPEC_CONST_RANGE.contains(&opcode) {
-                self.populate_one_spec_const(instr)?;
+                self.populate_one_spec_const(instr, cfg)?;
             } else { break; }
             instrs.next();
         }
@@ -1377,7 +1561,7 @@ impl<'a> ReflectIntermediate<'a> {
         itm.populate_names(&mut instrs)?;
         skip_until(&mut instrs, is_deco_op);
         itm.populate_decos(&mut instrs)?;
-        itm.populate_defs(&mut instrs)?;
+        itm.populate_defs(&mut instrs, cfg)?;
         itm.populate_access(&mut instrs, inspector)?;
         itm.collect_entry_points(cfg)
     }
@@ -1389,6 +1573,7 @@ pub struct ReflectConfig {
     spv: SpirvBinary,
     ref_all_rscs: bool,
     combine_img_samplers: bool,
+    spec_values: HashMap<SpecId, ConstantValue>,
 }
 impl ReflectConfig {
     pub fn new() -> Self { Default::default() }
@@ -1416,6 +1601,11 @@ impl ReflectConfig {
         self.combine_img_samplers = x;
         self
     }
+    /// Use the provided value for specialization constant at `spec_id`.
+    pub fn specialize(&mut self, spec_id: SpecId, value: ConstantValue) -> &mut Self {
+        self.spec_values.insert(spec_id, value);
+        self
+    }
 
     /// Reflect the SPIR-V binary and extract all entry points.
     pub fn reflect(&self) -> Result<Vec<EntryPoint>> {
@@ -1465,17 +1655,17 @@ impl<'a> ReflectIntermediate<'a> {
         // been refered to by the specified function. (Do we actually need this?
         // It might not be an optimization in mind of engineering.)
         let mut vars = Vec::new();
-        for spec_const in self.spec_consts.iter() {
-            let ty = self.get_ty(spec_const.ty_id)
-                .ok_or(Error::TY_NOT_FOUND)?;
-            let locator = Locator::SpecConstant(spec_const.spec_id);
-            let name = self.get_var_name(locator);
-            let spec = Variable::SpecConstant {
-                name: name.map(|x| x.to_owned()),
-                spec_id: spec_const.spec_id,
-                ty: ty.clone(),
-            };
-            vars.push(spec);
+        for constant in self.const_map.values() {
+            if let Some(spec_id) = constant.spec_id {
+                let locator = Locator::SpecConstant(spec_id);
+                let name = self.get_var_name(locator);
+                let spec = Variable::SpecConstant {
+                    name: name.map(|x| x.to_owned()),
+                    spec_id: spec_id,
+                    ty: constant.ty.clone(),
+                };
+                vars.push(spec);
+            }
         }
         Ok(vars)
     }
